package org.appfuse.web;

import java.beans.PropertyEditorSupport;
import org.springframework.util.StringUtils;
import org.appfuse.model.Address;

public class AddressEditor extends PropertyEditorSupport {
    /**
     * Parse the Address from the given text.
     * Assumes a format where the last comma separates the city and state and
     * the last space separates the state and zip
     */
    public void setAsText(String text) throws IllegalArgumentException {
        if (!StringUtils.hasText(text)) {
            // treat empty String as null value
            setValue(null);
        } else {
            String trim = text.trim();
            int lastComma = trim.lastIndexOf(",");
            int lastSpace = trim.lastIndexOf(" ");

            if ((lastComma >= 0) && (lastSpace >= 0) &&
                    (lastSpace > lastComma)) {
                Address address = new Address();
                address.setCity(trim.substring(0, lastComma));
                address.setState(trim.substring(lastComma + 1, lastSpace).trim());
                address.setZip(trim.substring(lastSpace + 1));
                setValue(address);
            } else {
                throw new IllegalArgumentException("Could not parse address: " +
                        text);
            }
        }
    }

    /**
     * Format the Address as a one-line String
     */
    public String getAsText() {
        Object value = getValue();

        if ((value != null) && (value instanceof Address)) {
            Address address = (Address) value;

            if (StringUtils.hasText(address.getCity())) {
                return address.getCity() + ", " + address.getState()
                + " " + address.getZip();
            }
        }

        return "";
    }
}
