package org.appfuse.web;

import java.io.Serializable;
import java.util.Comparator;

/**
 * A simple JavaBean to represent name-value pairs. 
 */
public class NameValue implements Comparable, Serializable {

    /**
     * Comparator that can be used for a case insensitive sort of
     * <code>NameValue</code> objects.
     */
    public static final Comparator CASE_INSENSITIVE_ORDER = new Comparator() {
        public int compare(Object o1, Object o2) {
            String name1 = ((NameValue) o1).getName();
            String name2 = ((NameValue) o2).getName();
            return name1.compareToIgnoreCase(name2);
        }
    };
    
    /**
     * The property which supplies the option name visible to the end user.
     */
    private String name = null;
    
    /**
     * The property which supplies the value returned to the server.
     */
    private String value = null;
    
    /**
     * Construct an instance with the supplied property values.
     *
     * @param name The name to be displayed to the user.
     * @param value The value to be returned to the server.
     */
    public NameValue(String name, String value) {
        this.name = name;
        this.value = value;
    }
    
    public String getName() {
        return this.name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getValue() {
        return this.value;
    }

    public void setValue(String value) {
        this.value = value;
    }

    /**
     * Compare NameValueBeans based on the name, because that's the human
     * viewable part of the object.
     *
     * @see Comparable
     */
    public int compareTo(Object o) {
        // Implicitly tests for the correct type, throwing
        // ClassCastException as required by interface
        String otherName = ((NameValue) o).getName();

        return this.getName().compareTo(otherName);
    }

    /**
     * Return a string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer("NameValue[");
        sb.append(this.name);
        sb.append(", ");
        sb.append(this.value);
        sb.append("]");
        return (sb.toString());
    }
}