package org.appfuse.web;

import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appfuse.model.Address;
import org.appfuse.model.Phone;
import org.appfuse.model.User;
import org.appfuse.service.UserManager;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.beans.propertyeditors.CustomNumberEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;


public class UserFormController extends SimpleFormController {
    private static Log log = LogFactory.getLog(UserFormController.class);
    private UserManager userManager = null;

    public void setUserManager(UserManager userManager) {
        this.userManager = userManager;
    }

    public UserFormController() {
        super();
        setCommandClass(User.class);
    }
    
    protected void onBind(HttpServletRequest request, Object o)
    throws Exception {
        User command = (User) o;

        // check if user entered a "type" on the "Add New" row
        String type = request.getParameter("new.phone.type");
        if (!"".equals(type)) {
            if (command.getPhones() == null) {
                command.setPhones(new HashSet());
            }

            String number = request.getParameter("new.phone.number");
            command.getPhones().add(new Phone(type, number));
        }
        
        // check for deleted phone numbers
        if (command.getPhones() != null) {
            String[] deletePhones = request.getParameterValues("deletePhone");
            Set phones = new HashSet(command.getPhones());
            for (int i = 0; deletePhones != null &&  i < deletePhones.length; i++) {
                String phoneId = deletePhones[i];
                for (Iterator it = phones.iterator(); it.hasNext();) {
                    Phone p = (Phone) it.next();
                    if (p.getId().toString().equals(phoneId)) {
                        command.getPhones().remove(p);
                    }
                }
            }
        }

        super.onBind(request, command);
    }

    protected Object formBackingObject(HttpServletRequest request)
    throws ServletException {
        String id = RequestUtils.getStringParameter(request, "id", "");
        User command = new User();

        if (!"".equals(id)) {
            command = userManager.getUser(id);
        }

        if (command.getAddress() == null) {
            command.setAddress(new Address());
        }

        return command;
    }

    protected void initBinder(HttpServletRequest request,
                              ServletRequestDataBinder binder) {
        // convert java.lang.Long
        NumberFormat nf = NumberFormat.getNumberInstance();
        binder.registerCustomEditor(Long.class,
                                    new CustomNumberEditor(Long.class, nf, true));

        // convert java.util.Date
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        dateFormat.setLenient(false);
        binder.registerCustomEditor(Date.class,
                                    new CustomDateEditor(dateFormat, true));

        // convert java.lang.Integer
        binder.registerCustomEditor(Integer.class,
                                    new CustomNumberEditor(Integer.class, nf,
                                                           true));

        // convert java.lang.Double
        binder.registerCustomEditor(Double.class,
                                    new CustomNumberEditor(Double.class, nf,
                                                           true));
        // convert org.appfuse.model.Address
        binder.registerCustomEditor(Address.class, new AddressEditor()); 
    }

    /**
     * Redirect to the successView when the cancel button has been pressed.
     */
    public ModelAndView processFormSubmission(HttpServletRequest request,
                                              HttpServletResponse response,
                                              Object command,
                                              BindException errors)
    throws Exception {
        if (request.getParameter("cancel") != null) {
            return new ModelAndView(getSuccessView());
        }

        return super.processFormSubmission(request, response, command, errors);
    }

    public ModelAndView onSubmit(HttpServletRequest request,
                                 HttpServletResponse response, Object command,
                                 BindException errors)
    throws Exception {
        if (log.isDebugEnabled()) {
            log.debug("entering 'onSubmit' method...");
        }

        User user = (User) command;

        if (request.getParameter("delete") != null) {
            userManager.removeUser(user.getId().toString());
            request.getSession().setAttribute("message",
                                              getMessageSourceAccessor()
                                                  .getMessage("user.deleted",
                                                              new Object[] {
                                                                  user.getFirstName() +
                                                                  ' ' +
                                                                  user.getLastName()
                                                              }));
        } else {
            userManager.saveUser(user);

            // put success message into session so it survives redirects
            request.getSession().setAttribute("message",
                                              getMessageSourceAccessor()
                                                  .getMessage("user.saved",
                                                              new Object[] {
                                                                  user.getFirstName() +
                                                                  ' ' +
                                                                  user.getLastName()
                                                              }));
        }

        return new ModelAndView(getSuccessView());
    }

    protected Map referenceData(HttpServletRequest request) {
        final Locale[] available = Locale.getAvailableLocales();

        List countries = new ArrayList();

        for (int i = 0; i < available.length; i++) {
            final String value = available[i].getCountry();
            final String name =
                available[i].getDisplayCountry(request.getLocale());

            if (!"".equals(value) && !"".equals(name)) {
                NameValue country = new NameValue(name, value);

                if (!countries.contains(country)) {
                    countries.add(new NameValue(name, value));
                }
            }
        }

        Collections.sort(countries, NameValue.CASE_INSENSITIVE_ORDER);

        Map countryMap = new HashMap();
        countryMap.put("countries", countries);

        return countryMap;
    }
}
