package org.appfuse.web;

import java.text.SimpleDateFormat;
import java.util.Date;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.appfuse.model.Address;
import org.appfuse.model.User;
import org.appfuse.service.UserManager;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.validation.BindException;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.AbstractWizardFormController;
import org.springframework.web.servlet.view.RedirectView;

public class UserWizardController extends AbstractWizardFormController {
    private static Log log = LogFactory.getLog(UserWizardController.class);
    private UserManager userManager;

    public UserWizardController() {
        setCommandClass(User.class);
    }

    public void setUserManager(UserManager userManager) {
        this.userManager = userManager;
    }

    protected ModelAndView processCancel(HttpServletRequest request,
                                         HttpServletResponse response,
                                         Object command, BindException errors)
    throws Exception {
        return new ModelAndView(new RedirectView("users.html"));
    }

    protected void initBinder(HttpServletRequest request,
                              ServletRequestDataBinder binder) {
        // convert java.util.Date
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        dateFormat.setLenient(false);
        binder.registerCustomEditor(Date.class,
                                    new CustomDateEditor(dateFormat, true));  
    }
    
    protected ModelAndView processFinish(HttpServletRequest request,
                                         HttpServletResponse response,
                                         Object command, BindException errors)
    throws Exception {
        if (log.isDebugEnabled()) {
            log.debug("executing 'processFinish'...");
        }

        User user = (User) command;
        userManager.saveUser(user);
        request.getSession().setAttribute("message", "User added successfully.");

        return new ModelAndView(new RedirectView("users.html"));
    }

    protected Object formBackingObject(HttpServletRequest request)
    throws Exception {
        User command = new User();
        command.setAddress(new Address());

        return command;
    }
}
