package org.appfuse.web.flow;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.appfuse.model.Address;
import org.appfuse.model.User;
import org.appfuse.service.UserManager;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.validation.DataBinder;
import org.springframework.web.flow.Event;
import org.springframework.web.flow.RequestContext;
import org.springframework.web.flow.ScopeType;
import org.springframework.web.flow.action.FormAction;


public class UserFormAction extends FormAction {
    private UserManager userManager;

    public void setUserManager(UserManager mgr) {
        this.userManager = mgr;
    }

    public UserFormAction() {
        setFormObjectName("user");
        setFormObjectClass(User.class);
        setFormObjectScope(ScopeType.FLOW);
    }

    public Object createFormObject(RequestContext context)
    throws InstantiationException, IllegalAccessException {
        User user = (User) super.createFormObject(context);
        user.setAddress(new Address());

        return user;
    }

    protected void initBinder(RequestContext context, DataBinder binder) {
        // convert java.util.Date
        SimpleDateFormat dateFormat = new SimpleDateFormat("MM/dd/yyyy");
        dateFormat.setLenient(false);
        binder.registerCustomEditor(Date.class,
                                    new CustomDateEditor(dateFormat, true));
    }
    
    public Event save(RequestContext context) throws Exception {
        User user = (User) context.getFlowScope().getAttribute("user");
        userManager.saveUser(user);

        return success();
    }
}
