package org.appfuse.dao;

import java.util.HashSet;
import java.util.Set;

import org.appfuse.model.Address;
import org.appfuse.model.Phone;
import org.appfuse.model.User;

public class UserDAOTest extends BaseDAOTestCase {
    private User user = null;
    private UserDAO dao = null;

    protected void setUp() throws Exception {
        super.setUp();
        dao = (UserDAO) ctx.getBean("userDAO");
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        dao = null;
    }

    public void testSaveUser() throws Exception {
        user = new User();
        user.setFirstName("Rod");
        user.setLastName("Johnson");

        dao.saveUser(user);
        assertNotNull("primary key assigned", user.getId());
        log.info(user);
        assertNotNull(user.getFirstName());
    }

    public void testAddAndRemoveUser() throws Exception {
        user = new User();
        user.setFirstName("Bill");
        user.setLastName("Joy");
        
        Address address = new Address();
        address.setCity("Denver");
        user.setAddress(address);
        
        Set phones = new HashSet();
        phones.add(new Phone("home", "303-555-1212"));
        phones.add(new Phone("work", "720-555-1212"));
        user.setPhones(phones);

        dao.saveUser(user);

        assertNotNull(user.getId());
        assertEquals(user.getFirstName(), "Bill");
        assertEquals("Denver", user.getAddress().getCity());
        assertEquals(2, user.getPhones().size());

        if (log.isDebugEnabled()) {
            log.debug("removing user...");
            log.info(user);
        }

        dao.removeUser(user.getId());

        assertNull(dao.getUser(user.getId()));
    }
}
